package bank;


/**
 * Abstract superclass of all representatives of accounts, i.e. both the
 * permanent account objects <code>AccountData</code> and the temporary proxy
 * objects <code>OfflineAccountProxy</code>. This class provides
 * infrastructure for storing lists of transactions and some standard
 * implementations of account methods.
 */
public abstract class Account {

    /**
     * Withdraw <code>amount</code> from the account
     * 
     * @param amount
     *            the amount to be withdrawn
     */
    public abstract void withdraw (int amount);

    /**
     * Try to withdraw <code>amount</code> from the account with checking
     * whether this is possible, return the amount of money that is given out
     * 
     * @param amount
     *            the amount to be withdrawn
     * @return <code>amount</code> if the withdrawal was successful,
     *         <code>0</code> otherwise
     */
    public int checkAndWithdraw (int amount) {
    }

    /**
     * Request a printed account statement. This request may fail at a late
     * point if the account is offline, in this case nothing happens (the
     * customer does not receive anything). For this reason the method does not
     * have any preconditions
     */
    public abstract void requestStatement ();

    /**
     * @return <code>true</code> iff the balance of this account can be
     *         determined (not possible for the offline situation)
     */
    public abstract boolean balanceIsAccessible ();

    /**
     * @return the balance of this account
     */
    public abstract int accountBalance ();

    /**
     * Determine whether a certain amount of money may be withdrawn. Using
     * method <code>PermanentAccount.withdraw(int,int)</code> it is possible to
     * circumvent this check, which is necessary because for offline withdrawals
     * the balance cannot be accessed and checked
     * 
     * @param amount
     *            the amount of money requested
     * @return <code>true</code> iff <code>amount</code> can be withdrawn
     *         from the account
     */
    public abstract boolean newWithdrawalIsPossible (int amount);

    /**
     * @return the account number of the account represented by
     *         <code>this</code> object
     */
    public int getAccountNumber () {
    }


    /**
     * Get the number of transactions for this account.
     * 
     * @return the number of transactions for this account
     */
    public  int getTransactionCount() {
    }

    /**
     * Get all transactions that have been performed for this account. In the
     * resulting array, the most recent transaction is the last element. Only
     * the first <code>getTransactionCount()</code> entries of the array are
     * valid, the remaining entries may have arbitrary content.
     * 
     * @return the array of transactions, some entries may be
     *          <code>null</code>
     */
    public Transaction[] getTransactions () {
    }
}
