package bank;

/**
 * Singleton class for representing an assumed system-wide clock (for simplicity
 * we suppose that both the central host and all ATMs are running on the same
 * clock).
 * @stereotype singleton
 */
public class Clock {

    private static final Clock clockInstance = new Clock();

    /** Duration of a day in clock ticks. */
    private static final int dayLength = 10;
    /** Duration of a month in clock ticks. */
    private static final int monthLength = 100;

    /**
     * The current date/time. We don't care about details of this representation
     *
     */
    private int currentDate = 0;

    /**
     * Private constructor as part of the applied singleton design pattern
     */
    private Clock() {}

    /**
     * @return Returns the instance.
     */
    public static Clock getInstance () {
        return clockInstance;
    }

    /**
     * @param date any date (as returned by <code>getCurrentDate()</code>)
     * @return the day of the month of <code>date</code>
     */
    public static int getDay (int date) {
        return ((date % monthLength) / dayLength) + 1;
    }

    /**
     * @param date any date (as returned by <code>getCurrentDate()</code>)
     * @return the month of <code>date</code>
     */
    public static int getMonth (int date) {
        return (date / monthLength) + 1;
    }

    /**
     * @return some date/time in a very distant past (used to initialise certain
     *         attributes in other classes)
     */
    public static int getBigBangsDate() {
        return -1000;
    }

    /**
     * @return <code>true</code> iff the two given dates refer to the same day
     */
    public static boolean isSameDay (int dateA, int dateB) {
        return isSameInterval ( dateA, dateB, dayLength );
    }

    /**
     * @return <code>true</code> iff <code>dateA</code> is an earlier date
     *         than <code>dateB</code>
     */
    public static boolean isEarlier (int dateA, int dateB) {
        return dateA < dateB;
    }

    /**
     * @return <code>true</code> iff the two given dates refer to the same month
     */
    public static boolean isSameMonth (int dateA, int dateB) {
        return isSameInterval ( dateA, dateB, monthLength );
    }

    /**
     * Check whether two dates belong to the same interval
     *
     * @param dateA some date (as returned by <code>getCurrentDate()</code>)
     * @param dateB some date (as returned by <code>getCurrentDate()</code>)
     * @param intervalLength length of the considered intervals (intervals
     *        run from <code>0</code> to <code>intervalLength-1</code>, from
     *        <code>intervalLength</code> to <code>2*intervalLength-1</code>
     *        and so on)
     * @return <code>true</code> iff the two given dates lie in the same
     *         interval
     */
    private static boolean isSameInterval (int dateA,
                                           int dateB,
                                           int intervalLength) {
        return dateA / intervalLength == dateB / intervalLength;
    }

    /**
     * Make the time progress (don't care about overflows ...)
     */
    public void tick () {
        currentDate++;
    }

    /**
     * @return the absolute current date/time
     */
    public int getCurrentDate () {
        return currentDate;
    }

    /**
     * @return the day of the month
     */
    public int getDay () {
        return getDay(currentDate);
    }

    /**
     * @return the month
     */
    public int getMonth () {
        return getMonth(currentDate);
    }
}
